"""
@file func_for_tests.py a set of functions, used in tests
to initialize the fields
"""
import numpy as np
sin = np.sin
import math
pi = math.pi


# 3D scalar field, no vectorization
def func_scal_1(res, x, y, z, t):
    res[0][...] = x - 0.1 * y + 10. * z * z * t
    return res


# 2D scalar field, vectorization required
def func_scal_2(x, y, z, t):
    f = x - 0.1 * y + 10. * z * z * t
    return f


# 3D vector field, no vectorization
def func_vec_1(res, x, y, z, t):
    res[0][...] = x
    res[1][...] = 0.1 * y
    res[2][...] = 10. * z * z
    return res


# 2D vector field, no vectorization
def func_vec_2(x, y, z, t):
    f_x = x
    f_y = 0.1 * y
    f_z = 10. * z * z
    return f_x, f_y, f_z


# 3D vector field, no vectorization, extra params
def func_vec_3(res, x, y, z, t, theta):
    res[0][...] = x + theta
    res[1][...] = 0.1 * y
    res[2][...] = 10. * z * z
    return res


# 3D vector field, vectorization required
def func_vec_4(x, y, z, t, theta):
    f_x = x + theta
    f_y = 0.1 * y
    f_z = 10. * z * z
    return f_x, f_y, f_z


# 3D field, 4 components, no vectorization, extra params
def func_vec_5(res, x, y, z, t, theta):
    res[0][...] = x + theta
    res[1][...] = 0.1 * y
    res[2][...] = 10. * z * z
    res[3][...] = theta * z
    return res


# 2D field, 4 components, no vectorization, extra param
def func_vec_6(res, x, y, t, theta):
    res[0][...] = x + theta
    res[1][...] = 0.1 * y
    res[2][...] = 10. * y
    res[3][...] = theta * t
    return res


def v3d(res, x, y, z, t):
    res[0][...] = sin(x) + t
    res[1][...] = y
    res[2][...] = z
    return res


def v3dbis(res, x, y, z, t):
    res[0][...] = 0.
    res[1][...] = sin(y)
    res[2][...] = t
    return res


def v2d(res, x, y, t):
    res[0][...] = sin(x) + t
    res[1][...] = y
    return res

